package cn.com.qmth.examcloud.web.support;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月14日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@ControllerAdvice
@ResponseBody
public class CustomExceptionHandler {

	/**
	 * 接口日志
	 */
	private static final ExamCloudLog INTERFACE_LOG = ExamCloudLogFactory
			.getLog("INTERFACE_LOGGER");

	private static ExamCloudLog LOG = ExamCloudLogFactory.getLog(CustomExceptionHandler.class);

	/**
	 * 不输出堆栈的状态码
	 */
	private static Set<String> nonStackTraceStatusCodes = new HashSet<>();

	static {
		String str = PropertiesUtil.getString("$log.nonStackTraceStatusCodes");
		if (StringUtils.isNotBlank(str)) {
			List<String> list = RegExpUtil.findAll(str, "[\\w\\-]+");
			CollectionUtils.addAll(nonStackTraceStatusCodes, list);
			LOG.info("nonStackTraceStatusCodes: " + JsonUtil.toJson(nonStackTraceStatusCodes));
		}
	}

	/**
	 * 异常处理
	 *
	 * @author WANGWEI
	 * @param e
	 * @param request
	 * @return
	 */
	@ExceptionHandler(RuntimeException.class)
	public ResponseEntity<StatusResponse> handleException(RuntimeException e,
			HttpServletRequest request) {

		Throwable cause = e.getCause();
		StatusResponse body = null;

		if (null != cause && cause instanceof StatusException) {
			StatusException se = (StatusException) cause;
			body = new StatusResponse(se.getCode(), se.getDesc());
		} else if (null != cause && cause instanceof DataIntegrityViolationException) {
			body = new StatusResponse(ResponseStatus.DATA_INTEGRITY_VIOLATION.getCode(),
					ResponseStatus.DATA_INTEGRITY_VIOLATION.getDesc());
		} else if (null != cause && cause instanceof IllegalStateException) {
			IllegalStateException se = (IllegalStateException) cause;
			body = new StatusResponse(ResponseStatus.FATAL_ERROR.getCode(), se.getMessage());
		} else {
			body = new StatusResponse(ResponseStatus.FATAL_ERROR.getCode(),
					ResponseStatus.FATAL_ERROR.getDesc());
			cause = e;
		}

		boolean alwaysOK = alwaysOK(request);
		if (alwaysOK) {
			INTERFACE_LOG.error("[HTTP-RESP]. StatusCode=" + HttpStatus.OK);
			if (nonStackTraceStatusCodes.contains(body.getCode())) {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body));
			} else {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body), cause);
			}
			return new ResponseEntity<StatusResponse>(body, HttpStatus.OK);
		} else {
			INTERFACE_LOG.error("[HTTP-RESP]. StatusCode=" + HttpStatus.INTERNAL_SERVER_ERROR);
			if (nonStackTraceStatusCodes.contains(body.getCode())) {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body));
			} else {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body), cause);
			}
			return new ResponseEntity<StatusResponse>(body, HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * 异常处理
	 *
	 * @author WANGWEI
	 * @param e
	 * @param request
	 * @return
	 */
	@ExceptionHandler(Exception.class)
	public ResponseEntity<StatusResponse> handleException(Exception e, HttpServletRequest request) {

		StatusResponse body = new StatusResponse(
				cn.com.qmth.examcloud.web.support.ResponseStatus.FATAL_ERROR.getCode(),
				cn.com.qmth.examcloud.web.support.ResponseStatus.FATAL_ERROR.getDesc());

		boolean alwaysOK = alwaysOK(request);
		if (alwaysOK) {
			INTERFACE_LOG.info("[HTTP-RESP]. StatusCode=" + HttpStatus.OK);
			if (nonStackTraceStatusCodes.contains(body.getCode())) {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body));
			} else {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body), e);
			}
			return new ResponseEntity<StatusResponse>(body, HttpStatus.OK);
		} else {
			INTERFACE_LOG.info("[HTTP-RESP]. StatusCode=" + HttpStatus.INTERNAL_SERVER_ERROR);
			if (nonStackTraceStatusCodes.contains(body.getCode())) {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body));
			} else {
				INTERFACE_LOG.error("[HTTP-RESP]. response=" + JsonUtil.toJson(body), e);
			}
			return new ResponseEntity<StatusResponse>(body, HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	/**
	 * 是否总是响应200
	 *
	 * @author WANGWEI
	 * @param request
	 * @return
	 */
	private boolean alwaysOK(HttpServletRequest request) {
		boolean alwaysOK = false;
		Object attribute = request.getAttribute("$ALWAYS_OK");
		if (null != attribute) {
			try {
				alwaysOK = (boolean) attribute;
			} catch (Exception ex) {
				// ignore
			}
		}
		return alwaysOK;
	}

}
